==> alicloud_impinteraction_sdk-1.8.6.tar.gz: android/src/main/java/com/aliyun/imp/flutter/sdk/alicloud_impinteraction_sdk/plugin/FPOfBase.java <==
package com.aliyun.imp.flutter.sdk.alicloud_impinteraction_sdk.plugin;

import android.app.Activity;
import android.util.Log;
import android.util.Pair;
import android.widget.Toast;

import androidx.annotation.CallSuper;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.lifecycle.Lifecycle;
import androidx.lifecycle.LifecycleObserver;
import androidx.lifecycle.OnLifecycleEvent;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.aliyun.imp.flutter.sdk.alicloud_impinteraction_sdk.FUtils;
import com.aliyun.imp.flutter.sdk.alicloud_impinteraction_sdk.inner.Holder;
import com.aliyun.roompaas.base.log.Logger;
import com.aliyun.roompaas.base.util.Utils;
import com.aliyun.roompaas.biz.exposable.RoomChannel;

import java.io.Serializable;
import java.lang.ref.Reference;
import java.lang.ref.WeakReference;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.embedding.engine.plugins.lifecycle.HiddenLifecycleReference;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.Result;

public abstract class FPOfBase extends FPAbstract implements LifecycleObserver {
    protected final String TAG;
    protected static final String K_RESULT = "result";
    protected static final String K_DETAIL = "detail";

    protected MethodChannel channel;
    protected Reference<Activity> activityRef;
    protected Reference<ActivityPluginBinding> actPluginBindingRef;
    protected Reference<Lifecycle> lifecycleRef;

    public interface Error {
        interface General {
            String CODE = "1000";
            String MSG = "";
        }

        interface LifecycleInvalid {
            String CODE = "1001";
            String MSG = "lifecycle invalid";
        }
    }

    public FPOfBase() {
        TAG = getClass().getSimpleName();
    }

    protected abstract String getMethodChannelName();

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {
        channel = new MethodChannel(binding.getBinaryMessenger(), getMethodChannelName());
        channel.setMethodCallHandler(this);
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        destroy();
    }

    @CallSuper
    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        Logger.i(TAG, "onMethodCall: method=" + call.method + ", args:" + call.arguments);
    }

    protected RoomChannel getRoomChannel() {
        return Holder.INSTANCE.roomChannel;
    }

    protected void postEvent(String event) {
        postEvent(event, null);
    }

    protected void postEvent(String event, Serializable serializable) {
        final Map<String, Object> params;
        if (serializable == null) {
            params = null;
        } else {
            String jsonStr = JSON.toJSONString(serializable);
            params = JSON.parseObject(jsonStr, new TypeReference<Map<String, Object>>() {
            }.getType());
        }
        FPOfEvent.postEventViaEventChannel(event, params);
    }

    public static String parseString(@NonNull MethodCall call, String key) {
        return parseString(call, key, "");
    }

    public static String parseString(@NonNull MethodCall call, String key, String defVal) {
        Object val = call.argument(key);
        return val != null ? String.valueOf(val) : defVal;
    }

    public static int parseInt(@NonNull MethodCall call, String key, int defVal) {
        try {
            return Integer.parseInt(Objects.requireNonNull(call.argument(key)));
        } catch (Throwable ignore) {
            return defVal;
        }
    }

    public static boolean parseBool(@NonNull MethodCall call, String key) {
        return Boolean.TRUE.equals(call.argument(key));
    }

    public static void takeSuccess(@NonNull Result result) {
        takeSuccess(result, " success");
    }

    public static void takeSuccess(@NonNull Result result, Object val) {
        takeSuccess(result, val, null);
    }

    @SuppressWarnings("unchecked")
    public static void takeSuccess(@NonNull Result result, Object val, @Nullable Object detail) {
        Logger.i(result + " success: val=" + val + ", detail=" + detail);
        try {
            if (detail == null) {
                result.success(asSimpleResult(val));
            } else {
                result.success(asM(new Pair<>(K_RESULT, String.valueOf(val)), new Pair<>(K_DETAIL, String.valueOf(detail))));
            }
        } catch (Throwable e) {
            Logger.e(e);
        }
    }

    public static void takeErrOfLifecycle(@NonNull Result result, String msgPrefix) {
        takeErr(result, Error.LifecycleInvalid.CODE, msgPrefix + "error :" + Error.LifecycleInvalid.MSG);
    }

    public static void takeGeneralErr(@NonNull Result result, String msg) {
        takeErr(result, Error.General.CODE, msg);
    }

    public static void takeErr(@NonNull Result result, String errorCode) {
        takeErr(result, errorCode, "");
    }

    public static void takeErr(@NonNull Result result, String errorCode, @Nullable String errorMessage) {
        takeErr(result, errorCode, errorMessage, null);
    }

    public static void takeErr(@NonNull Result result, String errorCode, @Nullable String errorMessage, @Nullable Object errorDetails) {
        Logger.e(result + " error: errorCode=" + errorCode + ", errorMessage=" + errorMessage);
        try {
            result.error(errorCode, errorMessage, errorDetails);
        } catch (Throwable e) {
            Logger.e(e);
        }
    }

    @SuppressWarnings("unchecked")
    public static Map<String, Object> asSimpleResult(@NonNull Object msg) {
        return asM(new Pair<>(K_RESULT, msg));
    }

    @SuppressWarnings("unchecked")
    @Nullable
    public static Map<String, Object> asM(Pair<String, Object>... ps) {
        if (ps == null || ps.length == 0) {
            return null;
        }

        Map<String, Object> m = new HashMap<>();
        for (Pair<String, Object> p : ps) {
            if (p == null) {
                continue;
            }
            m.put(p.first, String.valueOf(p.second));
        }
        return m;
    }

    protected void toast(String str) {
        Log.i(TAG, str);
        Activity activity = Utils.getRef(activityRef);
        if (Utils.isActivityInvalid(activity)) {
            Log.i(TAG, "end : invalid activity:" + activityRef);
            return;
        }

        Toast.makeText(activity, str, Toast.LENGTH_LONG).show();
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        activityRef = new WeakReference<>(binding.getActivity());
        actPluginBindingRef = new WeakReference<>(binding);
        Object lc = binding.getLifecycle();
        Logger.i(TAG, "onAttachedToActivity: " + lc);
        if (lc instanceof HiddenLifecycleReference) {
            HiddenLifecycleReference lcr = (HiddenLifecycleReference) lc;
            Lifecycle cycle = lcr.getLifecycle();
            cycle.addObserver(this);
            lifecycleRef = new WeakReference<>(cycle);
        }
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_RESUME)
    public void onResume() {
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_PAUSE)
    public void onPause() {
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_STOP)
    public void onStop() {
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
        actPluginBindingRef = new WeakReference<>(binding);
    }

    @Override
    public void onDetachedFromActivity() {
        destroy();
    }

    @Override
    public void destroy() {
        FUtils.removeLifecycleObserver(Utils.getRef(lifecycleRef), this);
        FUtils.clear(channel);
        Utils.clear(activityRef, actPluginBindingRef);
    }
}

==> alicloud_impinteraction_sdk-1.8.15.tar.gz: android/src/main/java/com/aliyun/imp/flutter/sdk/alicloud_impinteraction_sdk/plugin/FPOfBase.java <==
package com.aliyun.imp.flutter.sdk.alicloud_impinteraction_sdk.plugin;

import android.app.Activity;
import android.util.Log;
import android.util.Pair;
import android.widget.Toast;

import androidx.annotation.CallSuper;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.lifecycle.Lifecycle;
import androidx.lifecycle.LifecycleObserver;
import androidx.lifecycle.OnLifecycleEvent;

import com.aliyun.imp.flutter.sdk.alicloud_impinteraction_sdk.FUtils;
import com.aliyun.imp.flutter.sdk.alicloud_impinteraction_sdk.inner.Holder;
import com.aliyun.roompaas.base.log.Logger;
import com.aliyun.roompaas.base.util.Utils;
import com.aliyun.roompaas.biz.exposable.RoomChannel;

import java.io.Serializable;
import java.lang.ref.Reference;
import java.lang.ref.WeakReference;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.embedding.engine.plugins.lifecycle.HiddenLifecycleReference;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.Result;

public abstract class FPOfBase extends FPAbstract implements LifecycleObserver {
    protected final String TAG;
    protected static final String K_RESULT = "result";
    protected static final String K_DETAIL = "detail";

    protected MethodChannel channel;
    protected Reference<Activity> activityRef;
    protected Reference<ActivityPluginBinding> actPluginBindingRef;
    protected Reference<Lifecycle> lifecycleRef;

    public interface Error {
        interface General {
            String CODE = "1000";
            String MSG = "";
        }

        interface LifecycleInvalid {
            String CODE = "1001";
            String MSG = "lifecycle invalid";
        }
    }

    public FPOfBase() {
        TAG = getClass().getSimpleName();
    }

    protected abstract String getMethodChannelName();

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {
        channel = new MethodChannel(binding.getBinaryMessenger(), getMethodChannelName());
        channel.setMethodCallHandler(this);
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        destroy();
    }

    @CallSuper
    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        Logger.i(TAG, "onMethodCall: method=" + call.method + ", args:" + call.arguments);
    }

    protected RoomChannel getRoomChannel() {
        return Holder.INSTANCE.roomChannel;
    }

    protected void postEvent(String event) {
        postEvent(event, null);
    }

    protected void postEvent(String event, Serializable serializable) {
        FPOfEvent.postEventViaEventChannel(event, serializable);
    }

    public static String parseString(@NonNull MethodCall call, String key) {
        return parseString(call, key, "");
    }

    public static String parseString(@NonNull MethodCall call, String key, String defVal) {
        Object val = call.argument(key);
        return val != null ? String.valueOf(val) : defVal;
    }

    public static int parseInt(@NonNull MethodCall call, String key, int defVal) {
        try {
            return Integer.parseInt(Objects.requireNonNull(call.argument(key)));
        } catch (Throwable ignore) {
            return defVal;
        }
    }

    public static boolean parseBool(@NonNull MethodCall call, String key) {
        return Boolean.TRUE.equals(call.argument(key));
    }

    public static void takeSuccess(@NonNull Result result) {
        takeSuccess(result, " success");
    }

    public static void takeSuccess(@NonNull Result result, Object val) {
        takeSuccess(result, val, null);
    }

    @SuppressWarnings("unchecked")
    public static void takeSuccess(@NonNull Result result, Object val, @Nullable Object detail) {
        Logger.i(result + " success: val=" + val + ", detail=" + detail);
        try {
            if (detail == null) {
                result.success(asSimpleResult(val));
            } else {
                result.success(asM(new Pair<>(K_RESULT, String.valueOf(val)), new Pair<>(K_DETAIL, String.valueOf(detail))));
            }
        } catch (Throwable e) {
            Logger.e(e);
        }
    }

    public static void takeErrOfLifecycle(@NonNull Result result, String msgPrefix) {
        takeErr(result, Error.LifecycleInvalid.CODE, msgPrefix + "error :" + Error.LifecycleInvalid.MSG);
    }

    public static void takeGeneralErr(@NonNull Result result, String msg) {
        takeErr(result, Error.General.CODE, msg);
    }

    public static void takeErr(@NonNull Result result, String errorCode) {
        takeErr(result, errorCode, "");
    }

    public static void takeErr(@NonNull Result result, String errorCode, @Nullable String errorMessage) {
        takeErr(result, errorCode, errorMessage, null);
    }

    public static void takeErr(@NonNull Result result, String errorCode, @Nullable String errorMessage, @Nullable Object errorDetails) {
        Logger.e(result + " error: errorCode=" + errorCode + ", errorMessage=" + errorMessage);
        try {
            result.error(errorCode, errorMessage, errorDetails);
        } catch (Throwable e) {
            Logger.e(e);
        }
    }

    @SuppressWarnings("unchecked")
    public static Map<String, Object> asSimpleResult(@NonNull Object msg) {
        return asM(new Pair<>(K_RESULT, msg));
    }

    @SuppressWarnings("unchecked")
    @Nullable
    public static Map<String, Object> asM(Pair<String, Object>... ps) {
        if (ps == null || ps.length == 0) {
            return null;
        }

        Map<String, Object> m = new HashMap<>();
        for (Pair<String, Object> p : ps) {
            if (p == null) {
                continue;
            }
            m.put(p.first, String.valueOf(p.second));
        }
        return m;
    }

    protected void toast(String str) {
        Log.i(TAG, str);
        Activity activity = Utils.getRef(activityRef);
        if (Utils.isActivityInvalid(activity)) {
            Log.i(TAG, "end : invalid activity:" + activityRef);
            return;
        }

        Toast.makeText(activity, str, Toast.LENGTH_LONG).show();
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        activityRef = new WeakReference<>(binding.getActivity());
        actPluginBindingRef = new WeakReference<>(binding);
        Object lc = binding.getLifecycle();
        Logger.i(TAG, "onAttachedToActivity: " + lc);
        if (lc instanceof HiddenLifecycleReference) {
            HiddenLifecycleReference lcr = (HiddenLifecycleReference) lc;
            Lifecycle cycle = lcr.getLifecycle();
            cycle.addObserver(this);
            lifecycleRef = new WeakReference<>(cycle);
        }
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_RESUME)
    public void onResume() {
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_PAUSE)
    public void onPause() {
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_STOP)
    public void onStop() {
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
        actPluginBindingRef = new WeakReference<>(binding);
    }

    @Override
    public void onDetachedFromActivity() {
        destroy();
    }

    @Override
    public void destroy() {
        FUtils.removeLifecycleObserver(Utils.getRef(lifecycleRef), this);
        FUtils.clear(channel);
        Utils.clear(activityRef, actPluginBindingRef);
    }
}
